"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const mock = require("mock-fs");
const path = require("path");
const debug_1 = require("./debug");
const store_1 = require("./store");
const stringify_1 = require("./stringify");
const _1 = require(".");
describe('stringify()', () => {
    it('throws an Error if validation cannot convert to string', () => {
        const input = {
            Model: {
                attribute: {
                    validation: undefined
                }
            }
        };
        expect(() => { stringify_1.stringify(input); }).toThrow();
    });
    it('convert input to redprint', () => {
        const input = {
            Model: {
                attribute: {
                    validation: () => true
                }
            }
        };
        expect(stringify_1.stringify(input)).toEqual({
            Model: {
                attribute: {
                    validation: '() => true'
                }
            }
        });
    });
});
describe('debug()', () => {
    it('throws an Error if redprint is not object type', () => {
        const redprint = 'Hello!';
        expect(() => { debug_1.debug(redprint); }).toThrowError('Redprint must be an object');
    });
    it('throws an Error if some models are not object type', () => {
        const redprint = {
            Model: 'Hello!'
        };
        expect(() => { debug_1.debug(redprint); }).toThrowError('Model must be an object');
    });
    it('throws an Error if some attributes are not object type', () => {
        const redprint = {
            Model: {
                attribute: 'Hello'
            }
        };
        expect(() => { debug_1.debug(redprint); }).toThrowError('Attribute must be an object');
    });
    it('throws an Error if some validations are not string type', () => {
        const redprint = {
            Model: {
                attribute: {
                    validation: 1
                }
            }
        };
        expect(() => { debug_1.debug(redprint); }).toThrowError('Validation must be a string');
    });
    it('throws an Error if some validations are not convertable to function', () => {
        const redprint = {
            Model: {
                attribute: {
                    validation: '{}'
                }
            }
        };
        expect(() => { debug_1.debug(redprint); }).toThrowError('Validation must be convertable to function');
    });
    it('throws an Error if some validation has two or more arguments', () => {
        const redprint = {
            Model: {
                attribute: {
                    validation: '() => true'
                }
            }
        };
        expect(() => { debug_1.debug(redprint); }).toThrowError('Validation must have a single argument');
    });
    it('throws an Error if some validations do not return boolean type', () => {
        const redprint = {
            Model: {
                attribute: {
                    validation: "(i) => 'Hello!'"
                }
            }
        };
        expect(() => { debug_1.debug(redprint); }).toThrowError('Validation must return boolean');
    });
    it('does not throw an Error', () => {
        const redprint = {
            Model: {
                attribute: {
                    validation: '(i) => true'
                }
            }
        };
        expect(() => { debug_1.debug(redprint); }).not.toThrow();
    });
});
describe('store()', () => {
    afterEach(mock.restore);
    it('throws an Error if redprint.json is not valid', () => {
        mock({ 'redprint.json': 'Hello!' });
        const redprint = {};
        expect(() => { store_1.store(redprint); }).toThrow();
    });
    it('create redprint.json if not exists', () => {
        mock();
        const redprint = {};
        store_1.store(redprint);
        const data = fs.readJsonSync(path.join(process.cwd(), 'redprint.json'));
        expect(data).toEqual(redprint);
    });
    it('adds a model if not exist', () => {
        mock({
            'redprint.json': JSON.stringify({
                Model1: {}
            })
        });
        const redprint = {
            Model2: {}
        };
        store_1.store(redprint);
        const data = fs.readJsonSync(path.join(process.cwd(), 'redprint.json'));
        expect(data).toEqual({
            Model1: {},
            Model2: {}
        });
    });
    it('adds a attribute if not exist', () => {
        mock({
            'redprint.json': JSON.stringify({
                Model: {
                    attribute1: {}
                }
            })
        });
        const redprint = {
            Model: {
                attribute2: {}
            }
        };
        store_1.store(redprint);
        const data = fs.readJsonSync(path.join(process.cwd(), 'redprint.json'));
        expect(data).toEqual({
            Model: {
                attribute1: {},
                attribute2: {},
            }
        });
    });
    it('adds a validation if not exist', () => {
        mock({
            'redprint.json': JSON.stringify({
                Model: {
                    attribute: {
                        validation1: '() => true'
                    }
                }
            })
        });
        const redprint = {
            Model: {
                attribute: {
                    validation2: '() => true'
                }
            }
        };
        store_1.store(redprint);
        const data = fs.readJsonSync(path.join(process.cwd(), 'redprint.json'));
        expect(data).toEqual({
            Model: {
                attribute: {
                    validation1: '() => true',
                    validation2: '() => true',
                }
            }
        });
    });
});
describe('red()', () => {
    afterEach(mock.restore);
    it('returns the argument intactly if the argument is singular', () => {
        mock();
        const input = {};
        expect(_1.red(input)).toEqual(input);
    });
    it('returns the arguments as array if the arguments is multiple', () => {
        mock();
        const input = [{}, {}, {}];
        expect(_1.red(...input)).toEqual(input);
    });
    it('stores redprint at redprint.json', () => {
        mock();
        const input = {};
        _1.red(input);
        const data = fs.readJsonSync(path.join(process.cwd(), 'redprint.json'));
        expect(data).toEqual({});
    });
});
describe('getPurifier()', () => {
    it('returns a function', () => {
        expect(typeof _1.getPurifier()).toEqual('function');
    });
});
describe('setPurifier()', () => {
    it('sets a purifier', () => {
        const purifier = () => { };
        _1.setPurifier(purifier);
        expect(_1.getPurifier()).toEqual(purifier);
    });
});
//# sourceMappingURL=test.js.map